#!/bin/bash

ufftw="fftw"
vfftw="3.3.10"

sfftw="$ufftw-$vfftw"

cc=$(gcc -v 2>&1 | grep -o -E "i686|x86_64" -m 1)

if [ "$cc" == "i686" ]; then 
	arch="x32"
	confflags="--with-our-malloc16 --with-windows-f77-mangling --enable-shared --disable-static --enable-threads --with-combined-threads --with-incoming-stack-boundary=2"
	confflags_single=$confflags+" --enable-sse --enable-avx --enable-float"
elif [ "$cc" == "x86_64" ]; then 
	arch="x64"
	confflags="--disable-alloca --with-our-malloc16 --with-windows-f77-mangling --enable-shared --disable-static --enable-threads --with-combined-threads --with-incoming-stack-boundary=2"
	confflags_single=$confflags+" --enable-sse2 --enable-avx --enable-float"
fi
	confflags_double=$confflags+" --enable-sse2 --enable-avx"
	confflags_ldouble=$confflags+" --enable-long-double"

libs="/c/libs"
src="$libs/src/${arch}"

out="$libs/out/$ufftw/${arch}"
out_double="$out/double"
out_single="$out/single"
out_ldouble="$out/ldouble"

function make_build_fftw_double {

	set +o xtrace

#	cc="gcc -static-libgcc"
	cc="gcc -static-libgcc -fno-ident -g0 -O2"
 
	set -o xtrace

	cd "$out"
	rm -rf double
	mkdir double

	cd "$src/$sfftw"
	rm -rf @double
	mkdir @double
	cd @double

	../configure CC="$cc" ${confflags_double} --prefix="$out_double" \
		&& make clean > /dev/null \
		&& make -j4 > build.log \
		&& make install

	cp -f tests/.libs/bench.exe "$out_double/bin/fftw-bench.exe"

	set +o xtrace
}

function make_build_fftw_single {

	set +o xtrace

	cc="gcc -static-libgcc"
 
	set -o xtrace

	cd "$out"
	rm -rf single
	mkdir single

	cd "$src/$sfftw"
	rm -rf @single
	mkdir @single
	cd @single

	../configure CC="$cc" ${confflags_single} --prefix="$out_single" \
		&& make clean > /dev/null \
		&& make -j4 > build.log \
		&& make install

	cp -f tests/.libs/bench.exe "$out_single/bin/fftwf-bench.exe"

	set +o xtrace
}

function make_build_fftw_ldouble {

	set +o xtrace

	cc="gcc -static-libgcc"
 
	set -o xtrace

	cd "$out"
	rm -rf ldouble
	mkdir ldouble

	cd "$src/$sfftw"
	rm -rf @ldouble
	mkdir @ldouble
	cd @ldouble

	../configure CC="$cc" ${confflags_ldouble} --prefix="$out_ldouble" \
		&& make clean > /dev/null \
		&& make -j4 > build.log \
		&& make install

	cp -f tests/.libs/bench.exe "$out_ldouble/bin/fftwl-bench.exe"

	set +o xtrace
}

make_build_fftw_double
make_build_fftw_single
make_build_fftw_ldouble
